@@ -26,7 +26,6 @@ use rustyline::{
     Validator,
 };
 use rustyline::{EventHandler, Highlighter, KeyCode, KeyEvent, Modifiers};
-use toml;
 
 use std::io::IsTerminal;
 use std::path::PathBuf;
@@ -53,11 +52,14 @@ struct Args {
         short,
         long,
         value_name = "CODE",
-        conflicts_with = "file",
         action = clap::ArgAction::Append
     )]
     expression: Option<Vec<String>>,
 
+    /// Enter interactive session after running a numbat script or expression
+    #[arg(short, long)]
+    inspect_interactively: bool,
+
     /// Do not load the user configuration file.
     #[arg(long, hide_short_help = true)]
     no_config: bool,
@@ -133,11 +135,14 @@ impl Cli {
             Config::default()
         };
 
-        config.main.load_prelude &= !args.no_prelude;
-        config.main.load_user_init &= !(args.no_prelude || args.no_init);
+        config.load_prelude &= !args.no_prelude;
+        config.load_user_init &= !(args.no_prelude || args.no_init);
 
-        config.main.intro_banner = args.intro_banner.unwrap_or(config.main.intro_banner);
-        config.main.pretty_print = args.pretty_print.unwrap_or(config.main.pretty_print);
+        config.intro_banner = args.intro_banner.unwrap_or(config.intro_banner);
+        config.pretty_print = args.pretty_print.unwrap_or(config.pretty_print);
+
+        config.enter_repl =
+            (args.file.is_none() && args.expression.is_none()) || args.inspect_interactively;
 
         let mut fs_importer = FileSystemImporter::default();
         for path in Self::get_modules_paths() {
@@ -165,7 +170,7 @@ impl Cli {
     }
 
     fn run(&mut self) -> Result<()> {
-        if self.config.main.load_prelude {
+        if self.config.load_prelude {
             let result = self.parse_and_evaluate(
                 "use prelude",
                 CodeSource::Internal,
@@ -177,7 +182,7 @@ impl Cli {
             }
         }
 
-        if self.config.main.load_user_init {
+        if self.config.load_user_init {
             let user_init_path = Self::get_config_path().join("init.nbt");
 
             if let Ok(user_init_code) = fs::read_to_string(&user_init_path) {
@@ -193,55 +198,73 @@ impl Cli {
             }
         }
 
-        if self.config.main.load_prelude
+        if self.config.load_prelude
             && self.config.exchange_rates.fetching_policy != ExchangeRateFetchingPolicy::Never
         {
-            {
-                self.context
-                    .lock()
-                    .unwrap()
-                    .load_currency_module_on_demand(true);
-            }
-
-            if self.config.exchange_rates.fetching_policy == ExchangeRateFetchingPolicy::OnStartup {
-                thread::spawn(move || {
-                    numbat::Context::prefetch_exchange_rates();
-                });
-            }
+            self.context
+                .lock()
+                .unwrap()
+                .load_currency_module_on_demand(true);
         }
 
-        let code_and_source: Option<(String, CodeSource)> = if let Some(ref path) = self.file {
-            Some((
-                fs::read_to_string(path).context(format!(
+        let mut code_and_source = Vec::new();
+
+        if let Some(ref path) = self.file {
+            code_and_source.push((
+                (fs::read_to_string(path).context(format!(
                     "Could not load source file '{}'",
                     path.to_string_lossy()
-                ))?,
+                ))?),
                 CodeSource::File(path.clone()),
-            ))
-        } else {
-            self.expression
-                .as_ref()
-                .map(|exprs| (exprs.iter().join("\n"), CodeSource::Text))
+            ));
         };
 
-        if let Some((code, code_source)) = code_and_source {
-            let result = self.parse_and_evaluate(
-                &code,
-                code_source,
-                ExecutionMode::Normal,
-                self.config.main.pretty_print,
-            );
-
-            match result {
-                std::ops::ControlFlow::Continue(()) => Ok(()),
-                std::ops::ControlFlow::Break(ExitStatus::Success) => Ok(()),
-                std::ops::ControlFlow::Break(ExitStatus::Error) => {
-                    bail!("Interpreter stopped due to error")
-                }
-            }
-        } else {
-            self.repl()
+        if let Some(expressions) = &self.expression {
+            code_and_source.push((expressions.iter().join("\n"), CodeSource::Text));
         }
+
+        let mut run_result = Ok(());
+
+        if !code_and_source.is_empty() {
+            for (code, code_source) in code_and_source {
+                let result = self.parse_and_evaluate(
+                    &code,
+                    code_source,
+                    ExecutionMode::Normal,
+                    self.config.pretty_print,
+                );
+
+                let result_status = match result {
+                    std::ops::ControlFlow::Continue(()) => Ok(()),
+                    std::ops::ControlFlow::Break(_) => {
+                        bail!("Interpreter stopped")
+                    }
+                };
+
+                run_result = run_result.and(result_status);
+            }
+        }
+
+        if self.config.enter_repl {
+            let mut currency_fetch_thread = if self.config.load_prelude
+                && self.config.exchange_rates.fetching_policy
+                    == ExchangeRateFetchingPolicy::OnStartup
+            {
+                Some(thread::spawn(move || {
+                    numbat::Context::prefetch_exchange_rates();
+                }))
+            } else {
+                None
+            };
+
+            let repl_result = self.repl();
+            if let Some(thread) = currency_fetch_thread.take() {
+                let _ = thread.join();
+            }
+            run_result = run_result.and(repl_result);
+        }
+
+        run_result
     }
 
     fn repl(&mut self) -> Result<()> {
@@ -256,6 +279,7 @@ impl Cli {
             completer: NumbatCompleter {
                 context: self.context.clone(),
                 modules: self.context.lock().unwrap().list_modules().collect(),
+                all_timezones: chrono_tz::TZ_VARIANTS.map(|v| v.name()).into(),
             },
             highlighter: NumbatHighlighter {
                 context: self.context.clone(),
@@ -268,7 +292,7 @@ impl Cli {
         rl.load_history(&history_path).ok();
 
         if interactive {
-            match self.config.main.intro_banner {
+            match self.config.intro_banner {
                 IntroBanner::Long => {
                     println!();
                     println!(
@@ -306,7 +330,7 @@ impl Cli {
         interactive: bool,
     ) -> Result<()> {
         loop {
-            let readline = rl.readline(&self.config.main.prompt);
+            let readline = rl.readline(&self.config.prompt);
             match readline {
                 Ok(line) => {
                     if !line.trim().is_empty() {
@@ -370,7 +394,7 @@ impl Cli {
                                 println!();
                             }
                             _ => {
-                                if let Some(keyword) = line.strip_prefix("info") {
+                                if let Some(keyword) = line.strip_prefix("info ") {
                                     let help = self
                                         .context
                                         .lock()
@@ -387,7 +411,7 @@ impl Cli {
                                     } else {
                                         ExecutionMode::Normal
                                     },
-                                    self.config.main.pretty_print,
+                                    self.config.pretty_print,
                                 );
 
                                 match result {
@@ -473,6 +497,7 @@ impl Cli {
                     statements.last(),
                     &registry,
                     interactive || pretty_print,
+                    interactive || pretty_print,
                 );
                 print!("{}", ansi_format(&result_markup, false));
 
@@ -543,7 +568,7 @@ impl Cli {
         let data_dir = dirs::data_dir()
             .unwrap_or_else(|| PathBuf::from("."))
             .join("numbat");
-        fs::create_dir(&data_dir).ok();
+        fs::create_dir_all(&data_dir).ok();
         Ok(data_dir.join("history"))
     }
 }
